//============================================================+
// File name   : SunMoonPlanet.jsx
// Version     : 1.0.000
// Begin       : 2021-09-03
// Last Update : 2021-09-03
// Authors     : t.kenro - https://hoshifuru.jp - pr@hoshifuru.jp
// -------------------------------------------------------------------
// Copyright (C) 2021 t.kenro - hoshifuru.jp
//
//20210903 1.0.0 SunMoonPlanet.jsのv2.1.1を加筆修正。彗星のパラメーターをcomett.jsxに移す。惑星・彗星の日心黄経・日心黄緯を計算のe<1時のM計算の条件を変更。lookPlanetFromTheEarth()、calcPlanetPositionFromOrbitalElements()でJSONデータをパースせずに使用するよう変更。


//----------
//to calculate position of the Sun & the Moon
//参考文献：「日の出日の入りの計算」　長沢工　地人書館
//----------
//太陽黄経
function getSunPosition(Year,Month,Day,Hour,Minute,Second,isDst,timeZone){
	var T;
	var ls;

	T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDst,timeZone) / 365.25;
 	ls =	280.4603 + 360.00769 * T;
	ls +=	(1.9146 - 0.00005 * T)	* Math.sin(DegToRad(	357.538	+ 359.991	* T));
	ls +=	0.0200	* Math.sin(DegToRad(	355.05	+ 719.981	* T));
	ls +=	0.0048	* Math.sin(DegToRad(	234.95	+ 19.341	* T));
	ls +=	0.0020	* Math.sin(DegToRad(	247.1	+ 329.64	* T));
	ls +=	0.0018	* Math.sin(DegToRad(	297.8	+ 4452.67	* T));
	ls +=	0.0018	* Math.sin(DegToRad(	251.3	+ 0.20		* T));
	ls +=	0.0015	* Math.sin(DegToRad(	343.2	+ 450.37	* T));
	ls +=	0.0013	* Math.sin(DegToRad(	81.4	+ 225.18	* T));
	ls +=	0.0008	* Math.sin(DegToRad(	132.5	+ 659.29	* T));
	ls +=	0.0007	* Math.sin(DegToRad(	153.3	+ 90.38		* T));
	ls +=	0.0007	* Math.sin(DegToRad(	206.8	+ 30.35		* T));
	ls +=	0.0006	* Math.sin(DegToRad(	29.8	+ 337.18	* T));
	ls +=	0.0005	* Math.sin(DegToRad(	207.4	+ 1.50		* T));
	ls +=	0.0005	* Math.sin(DegToRad(	291.2	+ 22.81		* T));
	ls +=	0.0004	* Math.sin(DegToRad(	234.9	+ 315.56	* T));
	ls +=	0.0004	* Math.sin(DegToRad(	157.3	+ 299.30	* T));
	ls +=	0.0004	* Math.sin(DegToRad(	21.1	+ 720.02	* T));
	ls +=	0.0003	* Math.sin(DegToRad(	352.5	+ 1079.97	* T));
	ls +=	0.0003	* Math.sin(DegToRad(	329.7	+ 44.43		* T));
	while(ls < 0){ ls +=360; }
	ls = ls%360;

	return ls;
}

//地球と太陽の距離
function getREarthToSun(Year,Month,Day,Hour,Minute,Second,isDst,timeZone){
	var T;
	var rs;
	var q;
	T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDst,timeZone) / 365.25;

	q =	(0.007256 - 0.0000002 * T) * Math.sin(DegToRad(	267.54	+ 359.991	* T));
	q +=	0.000091  * Math.sin(DegToRad(	265.1	+ 719.98	* T));
	q +=	0.000030  * Math.sin(DegToRad(	90.0	+ 0		* T));
	q +=	0.000013  * Math.sin(DegToRad(	27.8	+ 4452.67	* T));
	q +=	0.000007  * Math.sin(DegToRad(	254	+ 450.4		* T));
	q +=	0.000007  * Math.sin(DegToRad(	156	+ 329.6		* T));
	rs = Math.pow(10,q);
	return rs;
}

//月の黄経
function getMoonPositionL(Year,Month,Day,Hour,Minute,Second,isDst,timeZone){
	var T;
	var Am;
	var lm;
	T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDst,timeZone) / 365.25;

	Am =	0.0040	* Math.sin(DegToRad(	119.5	+ 1.33	* T));
	Am +=	0.0020	* Math.sin(DegToRad(	55.0	+ 19.34	* T));
	Am +=	0.0006	* Math.sin(DegToRad(	71	+ 0.2	* T));
	Am +=	0.0006	* Math.sin(DegToRad(	54	+ 19.3	* T));

	lm =	218.3161 + 4812.67881 * T;
	lm +=	6.2887	* Math.sin(DegToRad(	134.961	+ 4771.9886	* T + Am));
	lm +=	1.2740	* Math.sin(DegToRad(	100.738	+ 4133.3536	* T));
	lm +=	0.6583	* Math.sin(DegToRad(	235.700	+ 8905.3422	* T));
	lm +=	0.2136	* Math.sin(DegToRad(	269.926	+ 9543.9773	* T));
	lm +=	0.1856	* Math.sin(DegToRad(	177.525	+ 359.9905	* T));
	lm +=	0.1143	* Math.sin(DegToRad(	6.546	+ 9664.0404	* T));
	lm +=	0.0588	* Math.sin(DegToRad(	214.22	+ 638.635	* T));
	lm +=	0.0572	* Math.sin(DegToRad(	103.21	+ 3773.363	* T));
	lm +=	0.0533	* Math.sin(DegToRad(	10.66	+ 13677.331	* T));
	lm +=	0.0459	* Math.sin(DegToRad(	238.18	+ 8545.352	* T));
	lm +=	0.0410	* Math.sin(DegToRad(	137.43	+ 4411.998	* T));
	lm +=	0.0348	* Math.sin(DegToRad(	117.84	+ 4452.671	* T));
	lm +=	0.0305	* Math.sin(DegToRad(	312.49	+ 5131.979	* T));
	lm +=	0.0153	* Math.sin(DegToRad(	130.84	+ 758.698	* T));
	lm +=	0.0125	* Math.sin(DegToRad(	141.51	+ 14436.029	* T));
	lm +=	0.0110	* Math.sin(DegToRad(	231.59	+ 4892.052	* T));
	lm +=	0.0107	* Math.sin(DegToRad(	336.44	+ 13038.696	* T));
	lm +=	0.0100	* Math.sin(DegToRad(	44.89	+ 14315.966	* T));
	lm +=	0.0085	* Math.sin(DegToRad(	201.5	+ 8266.71	* T));
	lm +=	0.0079	* Math.sin(DegToRad(	278.2	+ 4493.34	* T));
	lm +=	0.0068	* Math.sin(DegToRad(	53.2	+ 9265.33	* T));
	lm +=	0.0052	* Math.sin(DegToRad(	197.2	+ 319.32	* T));
	lm +=	0.0050	* Math.sin(DegToRad(	295.4	+ 4812.66	* T));
	lm +=	0.0048	* Math.sin(DegToRad(	235.0	+ 19.34		* T));
	lm +=	0.0040	* Math.sin(DegToRad(	13.2	+ 13317.34	* T));
	lm +=	0.0040	* Math.sin(DegToRad(	145.6	+ 18449.32	* T));
	lm +=	0.0040	* Math.sin(DegToRad(	119.5	+ 1.33		* T));
	lm +=	0.0039	* Math.sin(DegToRad(	111.3	+ 17810.68	* T));
	lm +=	0.0037	* Math.sin(DegToRad(	349.1	+ 5410.62	* T));
	lm +=	0.0027	* Math.sin(DegToRad(	272.5	+ 9183.99	* T));
	lm +=	0.0026	* Math.sin(DegToRad(	107.2	+ 13797.39	* T));
	lm +=	0.0024	* Math.sin(DegToRad(	211.9	+ 988.63	* T));
	lm +=	0.0024	* Math.sin(DegToRad(	252.8	+ 9224.66	* T));
	lm +=	0.0022	* Math.sin(DegToRad(	240.6	+ 8185.36	* T));
	lm +=	0.0021	* Math.sin(DegToRad(	87.5	+ 9903.97	* T));
	lm +=	0.0021	* Math.sin(DegToRad(	175.1	+ 719.98	* T));
	lm +=	0.0021	* Math.sin(DegToRad(	105.6	+ 3413.37	* T));
	lm +=	0.0020	* Math.sin(DegToRad(	55.0	+ 19.34		* T));
	lm +=	0.0018	* Math.sin(DegToRad(	4.1	+ 4013.29	* T));
	lm +=	0.0016	* Math.sin(DegToRad(	242.2	+ 18569.38	* T));
	lm +=	0.0012	* Math.sin(DegToRad(	339.0	+ 12678.71	* T));
	lm +=	0.0011	* Math.sin(DegToRad(	276.5	+ 19208.02	* T));
	lm +=	0.0009	* Math.sin(DegToRad(	218	+ 8586.0	* T));
	lm +=	0.0008	* Math.sin(DegToRad(	188	+ 14037.3	* T));
	lm +=	0.0008	* Math.sin(DegToRad(	204	+ 7906.7	* T));
	lm +=	0.0007	* Math.sin(DegToRad(	140	+ 4052.0	* T));
	lm +=	0.0007	* Math.sin(DegToRad(	275	+ 4853.3	* T));
	lm +=	0.0007	* Math.sin(DegToRad(	216	+ 278.6		* T));
	lm +=	0.0006	* Math.sin(DegToRad(	128	+ 1118.7	* T));
	lm +=	0.0005	* Math.sin(DegToRad(	247	+ 22582.7	* T));
	lm +=	0.0005	* Math.sin(DegToRad(	181	+ 19088.0	* T));
	lm +=	0.0005	* Math.sin(DegToRad(	114	+ 17450.7	* T));
	lm +=	0.0005	* Math.sin(DegToRad(	332	+ 5091.3	* T));
	lm +=	0.0004	* Math.sin(DegToRad(	313	+ 398.7		* T));
	lm +=	0.0004	* Math.sin(DegToRad(	278	+ 120.1		* T));
	lm +=	0.0004	* Math.sin(DegToRad(	71	+ 9584.7	* T));
	lm +=	0.0004	* Math.sin(DegToRad(	20	+ 720.0		* T));
	lm +=	0.0003	* Math.sin(DegToRad(	83	+ 3814.0	* T));
	lm +=	0.0003	* Math.sin(DegToRad(	66	+ 3494.7	* T));
	lm +=	0.0003	* Math.sin(DegToRad(	147	+ 18089.3	* T));
	lm +=	0.0003	* Math.sin(DegToRad(	311	+ 5492.0	* T));
	lm +=	0.0003	* Math.sin(DegToRad(	161	+ 40.7		* T));
	lm +=	0.0003	* Math.sin(DegToRad(	280	+ 23221.3	* T));

	while(lm < 0){ lm +=360; }
	lm = lm%360;
	return lm;
}

//月の黄緯
function getMoonPositionB(Year,Month,Day,Hour,Minute,Second,isDst,timeZone){
	var T;
	var bm;
	var Bm;
	T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDst,timeZone) / 365.25;

	Bm =	0.0267	* Math.sin(DegToRad(	234.95	+ 19.341	* T));
	Bm +=	0.0043	* Math.sin(DegToRad(	322.1	+ 19.36		* T));
	Bm +=	0.0040	* Math.sin(DegToRad(	119.5	+ 1.33		* T));
	Bm +=	0.0026	* Math.sin(DegToRad(	55.0	+ 19.34		* T));
	Bm +=	0.0005	* Math.sin(DegToRad(	307	+ 19.4		* T));

	bm =	5.1282	* Math.sin(DegToRad(	93.273	+ 4832.0202	* T + Bm));
	bm +=	0.2806	* Math.sin(DegToRad(	228.235	+ 9604.0088	* T));
	bm +=	0.2777	* Math.sin(DegToRad(	138.311	+ 60.0316	* T));
	bm +=	0.1732	* Math.sin(DegToRad(	142.427	+ 4073.3220	* T));
	bm +=	0.0554	* Math.sin(DegToRad(	194.01	+ 8965.374	* T));
	bm +=	0.0463	* Math.sin(DegToRad(	172.55	+ 698.667	* T));
	bm +=	0.0326	* Math.sin(DegToRad(	328.96	+ 13737.362	* T));
	bm +=	0.0172	* Math.sin(DegToRad(	3.18	+ 14375.997	* T));
	bm +=	0.0093	* Math.sin(DegToRad(	277.4	+ 8845.31	* T));
	bm +=	0.0088	* Math.sin(DegToRad(	176.7	+ 4711.96	* T));
	bm +=	0.0082	* Math.sin(DegToRad(	144.9	+ 3713.33	* T));
	bm +=	0.0043	* Math.sin(DegToRad(	307.6	+ 5470.66	* T));
	bm +=	0.0042	* Math.sin(DegToRad(	103.9	+ 18509.35	* T));
	bm +=	0.0034	* Math.sin(DegToRad(	319.9	+ 4433.31	* T));
	bm +=	0.0025	* Math.sin(DegToRad(	196.5	+ 8605.38	* T));
	bm +=	0.0022	* Math.sin(DegToRad(	331.4	+ 13377.37	* T));
	bm +=	0.0021	* Math.sin(DegToRad(	170.1	+ 1058.66	* T));
	bm +=	0.0019	* Math.sin(DegToRad(	230.7	+ 9244.02	* T));
	bm +=	0.0018	* Math.sin(DegToRad(	243.3	+ 8206.68	* T));
	bm +=	0.0018	* Math.sin(DegToRad(	270.8	+ 5192.01	* T));
	bm +=	0.0017	* Math.sin(DegToRad(	99.8	+ 14496.06	* T));
	bm +=	0.0016	* Math.sin(DegToRad(	135.7	+ 420.02	* T));
	bm +=	0.0015	* Math.sin(DegToRad(	211.1	+ 9284.69	* T));
	bm +=	0.0015	* Math.sin(DegToRad(	45.8	+ 9964.00	* T));
	bm +=	0.0014	* Math.sin(DegToRad(	219.2	+ 299.96	* T));
	bm +=	0.0013	* Math.sin(DegToRad(	95.8	+ 4472.03	* T));
	bm +=	0.0013	* Math.sin(DegToRad(	155.4	+ 379.35	* T));
	bm +=	0.0012	* Math.sin(DegToRad(	38.4	+ 4812.68	* T));
	bm +=	0.0012	* Math.sin(DegToRad(	148.2	+ 4851.36	* T));
	bm +=	0.0011	* Math.sin(DegToRad(	138.3	+ 19147.99	* T));
	bm +=	0.0010	* Math.sin(DegToRad(	18.0	+ 12978.66	* T));
	bm +=	0.0008	* Math.sin(DegToRad(	70	+ 17870.7	* T));
	bm +=	0.0008	* Math.sin(DegToRad(	326	+ 9724.1	* T));
	bm +=	0.0007	* Math.sin(DegToRad(	294	+ 13098.7	* T));
	bm +=	0.0006	* Math.sin(DegToRad(	224	+ 5590.7	* T));
	bm +=	0.0006	* Math.sin(DegToRad(	52	+ 13617.3	* T));
	bm +=	0.0005	* Math.sin(DegToRad(	280	+ 8485.3	* T));
	bm +=	0.0005	* Math.sin(DegToRad(	239	+ 4193.4	* T));
	bm +=	0.0004	* Math.sin(DegToRad(	311	+ 9483.9	* T));
	bm +=	0.0004	* Math.sin(DegToRad(	238	+ 23281.3	* T));
	bm +=	0.0004	* Math.sin(DegToRad(	81	+ 10242.6	* T));
	bm +=	0.0004	* Math.sin(DegToRad(	13	+ 9325.4	* T));
	bm +=	0.0004	* Math.sin(DegToRad(	147	+ 14097.4	* T));
	bm +=	0.0003	* Math.sin(DegToRad(	205	+ 22642.7	* T));
	bm +=	0.0003	* Math.sin(DegToRad(	107	+ 18149.4	* T));
	bm +=	0.0003	* Math.sin(DegToRad(	146	+ 3353.3	* T));
	bm +=	0.0003	* Math.sin(DegToRad(	234	+ 19268.0	* T));
	return bm;
}

//月の視差
function getMoonPositionPI(Year,Month,Day,Hour,Minute,Second,isDst,timeZone){
	var T;
	var PI;
	T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDst,timeZone) / 365.25;

	PI =	0.9507	* Math.sin(DegToRad(	90	));
	PI +=	0.0518	* Math.sin(DegToRad(	224.98	+ 4771.989		* T));
	PI +=	0.0095	* Math.sin(DegToRad(	190.7	+ 4133.35		* T));
	PI +=	0.0078	* Math.sin(DegToRad(	325.7	+ 8905.34		* T));
	PI +=	0.0028	* Math.sin(DegToRad(	0.0		+ 9543.98		* T));
	PI +=	0.0009	* Math.sin(DegToRad(	100.0	+ 13677.3		* T));
	PI +=	0.0005	* Math.sin(DegToRad(	329		+ 8545.4		* T));
	PI +=	0.0004	* Math.sin(DegToRad(	194		+ 3773.4		* T));
	PI +=	0.0003	* Math.sin(DegToRad(	227		+ 4412.0		* T));

	return PI;
}

//黄道傾斜角
function getEAngle(Year,Month,Day,Hour,Minute,Second,isDST,timeZone){
	var T = getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDST,timeZone)/365.25;
	var EAngle = 23.439291 - 0.000130042 * T;
	return EAngle;
}

//------------------
//地方恒星時と月齢
//参考文献：「日の出日の入りの計算」　長沢工　地人書館
//月齢の1～3次補正：t.kenro(hoshifuru.jp)による
//------------------
var LSThr = 0;
var moonAge = {	num: 0,	isRough: false	};

//地方恒星時・月齢へのアクセス関数
function getLSThr(){	return LSThr;	}
function getMoonAge(){	return moonAge.num;	}
function setLSThr(num){	LSThr = num;	}
function setMoonAge(num){	moonAge.num = num;	}
function moonAgeisRough(){	return 	moonAge.isRough;	}

//ユリウス通日の計算
function getJD(Year,Month,Day,Hour,Minute,Second,isDST,timeZone){
    var JDatUTC0;
    var JD;
    var YYYY = Year;
    var MM = Month;
    if(MM <= 2){
        YYYY = YYYY - 1;
        MM = MM + 12;
    }

    JDatUTC0 = Math.floor(365.25*YYYY) + Math.floor(YYYY/400) - Math.floor(YYYY/100) + Math.floor(30.59*(MM-2)) + Day + 1721088.5;
    JD = JDatUTC0 - timeZone/24 + Hour/24 + Minute/24/60 + Second/24/60/60;
    if(isDST){    JD = JD -1/24;    }

    return JD;
}

//J2000.0 2000-01-01 12:00:00（世界時）からの経過日数
function getDAYsfromJ2000(Year,Month,Day,Hour,Minute,Second,isDST,timeZone){
    var DAYsfromJ2000;
    var dT = 64;
    DAYsfromJ2000 = getJD(Year,Month,Day,Hour,Minute,Second,isDST,timeZone) - getJD(2000,1,1,12,0,0,false,0) + dT/86400;
    return DAYsfromJ2000;
}

//地方恒星時計算
function calkLSThr(YYYY,MM,DD,hh,mm,ss,isDST,timeZone,lon){
	var YDJ2000 = getDAYsfromJ2000(YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
	var T = YDJ2000/365.25;
//	var LST = 325.4606 + 360.007700536*T +0.00000003879*T*T + 360*(hh/24+mm/24/60+ss/24/60/60) + lon*1;
	var LST = 100.4606 + 360.007700536*T +0.00000003879*T*T + 360*(hh/24+mm/24/60+ss/24/60/60) - 15*timeZone + lon*1;
	if(isDST){	LST = LST -15;	}
	var LSThr = (LST/360*24)%24;
	if(LSThr<0){	LSThr += 24	};
	return LSThr;
}

//月齢計算
function calcMoonAge(YYYY,MM,DD,hh,mm,ss,isDST,timeZone){
	var Now = new Date(YYYY,MM-1,DD,hh,mm,ss);
	if(isDST){
		Now.setTime( Now.getTime() - 1*60*60*1000 );
	}

	var NewMoonInfo = {};
	NewMoonInfo = calcNewMoonTime(YYYY,MM,DD,hh,mm,ss,isDST,timeZone);

	var NewMoonDay = new Date(NewMoonInfo.YYYY,NewMoonInfo.MM-1,NewMoonInfo.DD,NewMoonInfo.hh,NewMoonInfo.mm,NewMoonInfo.ss);
	var mAge = (Now.getTime()- NewMoonDay.getTime())/(24*60*60*1000);
	var mAgeisRough = 0;

	//直前の新月日時を求められなかったときは、月齢の簡易計算式を用いる
	if(mAge < 0 || 29.85 <= mAge){
		mAge = calcNewMoonTimeRough(YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
		mAgeisRough = 1;
	}

	moonAge.isRough = mAgeisRough;
	return mAge;
}

//直前の新月時刻
function calcNewMoonTime(YYYY,MM,DD,hh,mm,ss,isDST,timeZone){
	var date = new Date(YYYY,MM-1,DD,hh,mm,ss);
	var g;
	var ls;
	var lm;
	var dl;
	var num = 0;

	do {
		ls = getSunPosition(date.getFullYear(),date.getMonth()+1,date.getDate(),date.getHours(),date.getMinutes(),date.getSeconds(),isDST,timeZone);
		lm = getMoonPositionL(date.getFullYear(),date.getMonth()+1,date.getDate(),date.getHours(),date.getMinutes(),date.getSeconds(),isDST,timeZone);

		dl = lm - ls;
		if(num == 0 && dl < 0){		dl +=360;	}
		if(num > 0 && dl < -330){	dl += 360;	}	
		if(num > 0 && dl > 330){	dl -= 360;	}
		date.setTime( date.getTime() - ( dl/12.1818 )*24*60*60*1000 );
		num++;
	} while(Math.abs(dl) > 0.001 && num < 30);

	if(isDST){	date.setTime( date.getTime() - 1*60*60*1000 );	}
	var NMT = {};
	NMT.YYYY	= date.getFullYear();
	NMT.MM		= date.getMonth()+1;
	NMT.DD		= date.getDate();
	NMT.hh		= date.getHours();
	NMT.mm		= date.getMinutes();
	NMT.ss		= date.getSeconds();
	return NMT;
}

//----------
//月齢近似式
//参考文献： おに・おに・にし-月齢簡易計算法 堀源一郎 http://www.asj.or.jp/geppou/archive_open/1968/pdf/19680704.pdf
//----------
function calcNewMoonTimeRough(YYYY,MM,DD,hh,mm,ss,isDST,timeZone){
	var c = [0,2,0,2,2,4,5,6,7,8,9,10];
	var mAge = ( (YYYY-11)%19*11 + c[MM-1] + DD + (hh)/24 + mm/24/60 + ss/24/60/60 )%30;
	return Math.round(mAge*1000)/1000;
}

//----------
//ケプラー方程式を解く
//Newton-Raphson method
//参考文献：JPL Keplerian Elements for Approximate Positions of the Major Planets http://ssd.jpl.nasa.gov/?planet_pos
//参考文献：The hyperbolic Kepler equation (and the elliptic equation revisited) Gooding, R. H. & Odell, A. W. http://adsabs.harvard.edu/full/1988CeMec..44..267G
//----------
function solveKeplersEquation(e,M){
	if(e<1){		return solveKeplersEquationOfEllipse(e,M);		//e<1
	}else if(1<e){	return solveKeplersEquationOfHyperbolic(e,M);	//1<e
	}else{			return solveKeplersEquationOfParabola(e,M);		//e=1
	}
}

//楕円軌道(e<1)
function solveKeplersEquationOfEllipse(e,M){
	var edeg = e / Math.PI *180;
	var E = M + edeg*Math.sin(DegToRad(M));
	var dM = 0;
	var dE = 0;
	do{
		E = E + dE;
		dM = M - (E - edeg * Math.sin(DegToRad(E)));
		dE = dM/(1-e * Math.cos(DegToRad(E)));
	}while(Math.abs(dE) > 0.000001 );
	return DegToRad(E);
}

//双曲線軌道(1<e)
function solveKeplersEquationOfHyperbolic(e,M){
	var dE = 0;
//	var E = M/(e-1);
	var E = 0.1*M/(e-1);
	do{
		E = E + dE;
		dE = (M-e*sinh(E) + E)/(e*cosh(E)-1);
	}while(Math.abs(dE) > 0.000001 );
//	E = E + dE;
	return E;
}

//放物線軌道(e=1)
function solveKeplersEquationOfParabola(e,M){	//you can not use this function yet
	return 0;
}

//惑星の軌道要素を計算
function calcPlanetElement(pData,YYYY,MM,DD,hh,mm,ss,isDST,timeZone){
	var planetNow = {
		name : "",	name_ja : "",
		a : 0,	e : 0,	I : 0,	L : 0,	lp: 0,	ln: 0,
		M : 0,	w : 0,	E : 0,	v : 0,	rEtoP : 0,	vlp : 0,	vbp : 0,
		X : 0,	Y : 0,	Z : 0,	rStoP : 0, scl : 0,	scb : 0
	}

	var Tc = getDAYsfromJ2000(YYYY,MM,DD,hh,mm,ss,isDST,timeZone) /36525;
	var Te = getDAYsfromJ2000(2000,1,1,12,0,0,false,0) /36525;
	var T = Tc-Te;

	planetNow.name	= pData.name;
	planetNow.name_ja	= pData.name_ja;
	planetNow.a	= pData.a0 + pData.da * T;
	planetNow.e	= pData.e0 + pData.de * T;
	planetNow.I	= pData.I0 + pData.dI * T;
	planetNow.L	= pData.L0 + pData.dL * T;
	planetNow.lp	= pData.lp0 + pData.dlp * T;
	planetNow.ln	= pData.ln0 + pData.dln * T;
	planetNow.w	= planetNow.lp - planetNow.ln;
	planetNow.M	= planetNow.L - planetNow.lp + pData.b*Math.pow(T,2) + pData.c*Math.cos(pData.f*T) + pData.s*Math.sin(pData.f*T);

//	while(planetNow.a < 0){		planetNow.a += 360; }
//	while(planetNow.e < 0){		planetNow.e += 360; }
	while(planetNow.I < 0){		planetNow.I += 360; }
	while(planetNow.L < 0){		planetNow.L += 360; }
	while(planetNow.lp < 0){	planetNow.lp += 360; }
	while(planetNow.ln < 0){	planetNow.ln += 360; }
	while(planetNow.M < -180){	planetNow.M += 360; }
	while(planetNow.M > 180){	planetNow.M -= 360; }
	while(planetNow.w < 0){		planetNow.w += 360; }

//	planetNow.a = planetNow.a % 360;
//	planetNow.e = planetNow.e % 360;
	planetNow.I = planetNow.I % 360;
	planetNow.L = planetNow.L % 360;
	planetNow.lp = planetNow.lp % 360;
	planetNow.ln = planetNow.ln % 360;
	planetNow.M = planetNow.M % 360;
	planetNow.w = planetNow.w % 360;

	return planetNow;
}

function getAddPlanetStr(YYYY,MM,DD,hh,mm,ss,isDST,timezone){
	//Wed Jan 01 1800 09:00:00 GMT+0918 (日本標準時)     -5364663539000
	//Sat Jan 01 2050 09:00:00 GMT+0900 (日本標準時)     2524608000000
	//Sun Jan 01 2051 09:00:00 GMT+0900 (日本標準時)     2556144000000
	var calcDate =  new Date(YYYY,MM-1,DD,hh,mm,ss);
	var addPlanetStr = (-5364663539000 < calcDate.getTime() && calcDate.getTime() < 2556144000000)? "_1800AD_2050AD" : "";
//	var addPlanetStr = (-5364663539000 < calcDate.getTime() && calcDate.getTime() < 2524608000000)? "_1800AD_2050AD" : "";
//	var addPlanetStr = (-5364663539000 < calcDate.getTime() && calcDate.getTime() < 2524608000000)? "" : "";

	return addPlanetStr;
}

function getEarthPosition(earthData,YYYY,MM,DD,hh,mm,ss,isDST,timezone){
	var earthNow	= calcPlanetlb_lookFromSun(earthData,YYYY,MM,DD,hh,mm,ss,isDST,timezone);
	earthNow.lsun	= (earthNow.scl+180)%360;	//太陽黄経
	earthNow.rStoE	= earthNow.rStoP;			//太陽-地球間距離

	return earthNow;
}

//惑星の日心黄経・日心黄緯（主に、地球に用いる）
function calcPlanetlb_lookFromSun(pData,YYYY,MM,DD,hh,mm,ss,isDST,timeZone){
	var OrbitalElements = {};
	OrbitalElements = calcPlanetElement(pData,YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
	return calcPlanetPositionFromOrbitalElements(OrbitalElements,YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
}

//惑星の視黄経・視黄緯
function calcPlanetlb_lookFromEarth(pData,YYYY,MM,DD,hh,mm,ss,isDST,timeZone,rStoE,lsun){
	var OrbitalElements = {};
	OrbitalElements = calcPlanetlb_lookFromSun(pData,YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
	return lookPlanetFromTheEarth(OrbitalElements,rStoE,lsun);
}

//惑星・彗星の日心黄経・日心黄緯を計算
function calcPlanetPositionFromOrbitalElements(OrbitalElements,YYYY,MM,DD,hh,mm,ss,isDST,timeZone){

//	var oElem = JSON.parse(JSON.stringify(OrbitalElements));
	var oElem = OrbitalElements;
	var planetXYZ = {	X: 0,	Y: 0,	Z: 0	}
	if(oElem.e<1){
//		if(typeof oElem.M === "undefined"){
		if(typeof oElem.Tp !== "undefined"){
			var JDOfTp = getDAYsfromJ2000(oElem.Tp.YYYY,oElem.Tp.MM,Math.floor(oElem.Tp.DD),0,0,0,false,0);		
			JDOfTp += oElem.Tp.DD % 1;
			var JDOfT = getDAYsfromJ2000(YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
			oElem.M = oElem.n * (JDOfT - JDOfTp);
		};

		oElem.E = solveKeplersEquation(oElem.e,oElem.M);
		planetXYZ = {
			X: oElem.a * ( Math.cos(oElem.E) - oElem.e),
			Y: oElem.a * Math.sqrt(1-Math.pow(oElem.e,2)) * Math.sin(oElem.E),
			Z: 0
		}
	}else if(1<oElem.e){
		var Te = getDAYsfromJ2000(oElem.Tp.YYYY,oElem.Tp.MM,oElem.Tp.DD,0,0,0,false,0);
		var Tc = getDAYsfromJ2000(YYYY,MM,DD,hh,mm,ss,isDST,timeZone);
		var K=0.000295912;
		var a=oElem.q/(oElem.e-1);
		var n=Math.sqrt(K)/a/Math.sqrt(a);

		oElem.E = solveKeplersEquation(oElem.e,n*(Tc-Te));
		planetXYZ = {
			X: 1 * a*(oElem.e-cosh(oElem.E)),
			Y: 1 * a*Math.sqrt(Math.pow(oElem.e,2)-1)*sinh(oElem.E),
			Z: 0
		}
	}

	//adjust object's orbit
	planetXYZ = rotateZ(planetXYZ.X,planetXYZ.Y,planetXYZ.Z,oElem.w);
	planetXYZ = rotateX(planetXYZ.X,planetXYZ.Y,planetXYZ.Z,oElem.I);
	planetXYZ = rotateZ(planetXYZ.X,planetXYZ.Y,planetXYZ.Z,oElem.ln);

	//太陽-惑星間距離 rStoP [天文単位]
	oElem.rStoP = Math.sqrt(Math.pow(planetXYZ.X,2)+Math.pow(planetXYZ.Y,2)+Math.pow(planetXYZ.Z,2));

	oElem.X = planetXYZ.X;
	oElem.Y = planetXYZ.Y;	
	oElem.Z = planetXYZ.Z;	

	planetXYZ.X = planetXYZ.X/oElem.rStoP;
	planetXYZ.Y = planetXYZ.Y/oElem.rStoP;
	planetXYZ.Z = planetXYZ.Z/oElem.rStoP;

	//日心黄経 scl 日心黄緯 scb
	oElem.scl = RadToDeg(Math.atan2(planetXYZ.Y,planetXYZ.X));
	oElem.scb = RadToDeg(Math.asin(planetXYZ.Z));

	return oElem;
}

function lookPlanetFromTheEarth(OrbitalElements,rStoE,ls){
//	var oElem = JSON.parse(JSON.stringify(OrbitalElements));
	var oElem = OrbitalElements;

	var planetXYZ = {
		X: oElem.X,
		Y: oElem.Y,
		Z: oElem.Z
	}

	//look object from the earth
	planetXYZ.X = planetXYZ.X + rStoE * Math.cos(DegToRad(ls));
	planetXYZ.Y = planetXYZ.Y + rStoE * Math.sin(DegToRad(ls));
	planetXYZ.Z = planetXYZ.Z;

	//地球-惑星間距離 rEtoP [天文単位]
	oElem.rEtoP = Math.sqrt(Math.pow(planetXYZ.X,2)+Math.pow(planetXYZ.Y,2)+Math.pow(planetXYZ.Z,2));

	planetXYZ.X = planetXYZ.X/oElem.rEtoP;
	planetXYZ.Y = planetXYZ.Y/oElem.rEtoP;
	planetXYZ.Z = planetXYZ.Z/oElem.rEtoP;

	//視黄経 vlp 視黄緯 vbp
	oElem.vlp = RadToDeg(Math.atan2(planetXYZ.Y,planetXYZ.X));
	oElem.vbp = RadToDeg(Math.asin(planetXYZ.Z));

	return oElem;
}

//----------
//惑星データ
//参考文献：JPL Keplerian Elements for Approximate Positions of the Major Planets http://ssd.jpl.nasa.gov/?planet_pos
//----------
var Sun = {
	name : "Sun",		name_ja: "太陽",
}
var Moon = {
	name : "Moon",		name_ja: "月",
}
var Mercury = {
	name : "Mercury",		name_ja: "水星",
	a0 : 0.38709843,	e0 : 0.20563661,	I0 : 7.00559432,	L0 : 252.25166724,	lp0: 77.45771895,	ln0: 48.33961819,	
	da : 0.00000000,	de : 0.00002123,	dI : -0.00590158,	dL : 149472.67486623,	dlp: 0.15940013,	dln: -0.12214182,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Venus = {
	name : "Venus",		name_ja: "金星",
	a0 : 0.72332102,	e0 : 0.00676399,	I0 : 3.39777545,	L0 : 181.97970850,	lp0: 131.76755713,	ln0: 76.67261496,	
	da : -0.00000026,	de : -0.00005107,	dI : 0.00043494,	dL : 58517.81560260,	dlp: 0.05679648,	dln: -0.27274174,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Earth = {
	name : "Earth",		name_ja: "地球",
	a0 : 1.00000018,	e0 : 0.01673163,	I0 : -0.00054346,	L0 : 100.46691572,	lp0: 102.93005885,	ln0: -5.11260389,
	da : -0.00000003,	de : -0.00003661,	dI : -0.01337178,	dL : 35999.37306329,	dlp: 0.31795260,	dln: -0.24123856,
	b : 0,	c : 0,		s : 0,	f : 0
}
var Mars = {
	name : "Mars",		name_ja: "火星",
	a0 : 1.52371243,	e0 : 0.09336511,	I0 : 1.85181869,	L0 : -4.56813164,	lp0: -23.91744784,	ln0: 49.71320984,
	da : 0.00000097,	de : 0.00009149,	dI : -0.00724757,	dL : 19140.29934243,	dlp: 0.45223625,	dln: -0.26852431,
	b : 0,	c : 0,		s : 0,	f : 0
}
var Jupiter = {
	name : "Jupiter",		name_ja: "木星",
	a0 : 5.20248019,	e0 : 0.04853590,	I0 : 1.29861416,	L0 : 34.33479152,	lp0: 14.27495244,	ln0: 100.29282654,
	da : -0.00002864,	de : 0.00018026,	dI : -0.00322699,	dL : 3034.90371757,	dlp: 0.18199196,	dln: 0.13024619,
	b : -0.00012452,	c : 0.06064060,		s : -0.35635438,	f : 38.35125000
}
var Saturn = {
	name : "Saturn",		name_ja: "土星",
	a0 : 9.54149883,	e0 : 0.05550825,	I0 : 2.49424102,	L0 : 50.07571329,	lp0: 92.86136063,	ln0: 113.63998702,
	da : -0.00003065,	de : -0.00032044,	dI : 0.00451969,	dL : 1222.11494724,	dlp: 0.54179478,	dln: -0.25015002,
	b : 0.00025899,		c : -0.13434469,	s : 0.87320147,		f : 38.35125000
}
var Uranus = {
	name : "Uranus",		name_ja: "天王星",
	a0 : 19.18797948,	e0 : 0.04685740,	I0 : 0.77298127,	L0 : 314.20276625,	lp0: 172.43404441,	ln0: 73.96250215,
	da : -0.00020455,	de : -0.00001550,	dI : -0.00180155,	dL : 428.49512595,	dlp: 0.09266985,	dln: 0.05739699,
	b : 0.00058331,		c : -0.97731848,	s : 0.17689245,		f : 7.67025000
}
var Neptune = {
	name : "Neptune",		name_ja: "海王星",
	a0 : 30.06952752,	e0 : 0.00895439,	I0 : 1.77005520,	L0 : 304.22289287,	lp0: 46.68158724,	ln0: 131.78635853,
	da : 0.00006447,	de : 0.00000818,	dI : 0.00022400,	dL : 218.46515314,	dlp: 0.01009938,	dln: -0.00606302,
	b : -0.00041348,	c : 0.68346318,		s : -0.10162547,	f : 7.67025000
}


//惑星データ（1800 AD - 2050 AD）
var Mercury_1800AD_2050AD = {
	name : "Mercury",		name_ja: "水星",
	a0 : 0.38709927,	e0 : 0.20563593,	I0 : 7.00497902,	L0 : 252.25032350,		lp0: 77.45779628,	ln0: 48.33076593,	
	da : 0.00000037,	de : 0.00001906,	dI : -0.00594749,	dL : 149472.67411175,	dlp: 0.16047689,	dln: -0.12534081,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Venus_1800AD_2050AD = {
	name : "Venus",		name_ja: "金星",
	a0 : 0.72333566,	e0 : 0.00677672,	I0 : 3.39467605,	L0 : 181.97909950,		lp0: 131.60246718,	ln0: 76.67984255,	
	da : 0.00000390,	de : -0.00004107,	dI : -0.00078890,	dL : 58517.81538729,	dlp: 0.00268329,	dln: -0.27769418,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Earth_1800AD_2050AD = {
	name : "Earth",		name_ja: "地球",
	a0 : 1.00000261,	e0 : 0.01671123,	I0 : -0.00001531,	L0 : 100.46457166,		lp0: 102.93768193,	ln0: 0.0,	
	da : 0.00000562,	de : -0.00004392,	dI : -0.01294668,	dL : 35999.37244981,	dlp: 0.32327364,	dln: 0.0,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Mars_1800AD_2050AD = {
	name : "Mars",		name_ja: "火星",
	a0 : 1.52371034,	e0 : 0.09339410,	I0 : 1.84969142,	L0 : -4.55343205,		lp0: -23.94362959,	ln0: 49.55953891,	
	da : 0.00001847,	de : 0.00007882,	dI : -0.00813131,	dL : 19140.30268499,	dlp: 0.44441088,	dln: -0.29257343,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Jupiter_1800AD_2050AD = {
	name : "Jupiter",		name_ja: "木星",
	a0 : 5.20288700,	e0 : 0.04838624,	I0 : 1.30439695,	L0 : 34.39644051,	lp0: 14.72847983,	ln0: 100.47390909,	
	da : -0.00011607,	de : -0.00013253,	dI : -0.00183714,	dL : 3034.74612775,	dlp: 0.21252668,	dln: 0.20469106,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Saturn_1800AD_2050AD = {
	name : "Saturn",		name_ja: "土星",
	a0 : 9.53667594,	e0 : 0.05386179,	I0 : 2.48599187,	L0 : 49.95424423,	lp0: 92.59887831,	ln0: 113.66242448,	
	da : -0.00125060,	de : -0.00050991,	dI : 0.00193609,	dL : 1222.49362201,	dlp: -0.41897216,	dln: -0.28867794,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Uranus_1800AD_2050AD = {
	name : "Uranus",		name_ja: "天王星",
	a0 : 19.18916464,	e0 : 0.04725744,	I0 : 0.77263783,	L0 : 313.23810451,	lp0: 170.95427630,	ln0: 74.01692503,	
	da : -0.00196176,	de : -0.00004397,	dI : -0.00242939,	dL : 428.48202785,	dlp: 0.40805281,	dln: 0.04240589,	
	b : 0,	c : 0,		s : 0,	f : 0
}
var Neptune_1800AD_2050AD = {
	name : "Neptune",		name_ja: "海王星",
	a0 : 30.06992276,	e0 : 0.00859048,	I0 : 1.77004347,	L0 : -55.12002969,	lp0: 44.96476227,	ln0: 131.78422574,	
	da : 0.00026291,	de : 0.00005105,	dI : 0.00035372,	dL : 218.45945325,	dlp: -0.32241464,	dln: -0.00508664,	
	b : 0,	c : 0,		s : 0,	f : 0
}
